package require Url 0
package require net

set menubar [frame .omfMenuBar -borderwidth 2 -relief raised]

# File menu
set fmb [menubutton .omfFile -text "File" -menu .omfFile.menu -underline 0]
set fmenu [menu .omfFile.menu -tearoff 0]
$fmenu add command -label "Exit" -underline 1 -command CleanupAndExit

# Help menu
set hmb [menubutton .omfHelp -text "Help" -menu .omfHelp.menu -underline 0]
set hmenu [menu .omfHelp.menu -tearoff 0]
$hmenu add command -label "About" -underline 0 -state disabled
$hmenu add command -label "On Oommf..." \
   -underline 3 -command [list OMFHB_Browser \
   [OMFURL_FileToUrl [file join $omf_library \
   [OMF_FileNameBase][OMF_Version] help]]]

proc DisplayHostButtons {} {
    global hframe hpanel
    # Destroy existing buttons
    foreach w [winfo children $hframe] {
        destroy $w
    }
    foreach host [net_host instances] {
        if {[$host Ready]} {
            set btn [checkbutton $hframe.w$host -text [$host cget -hostname] \
                -variable  ${host}_BtnVar \
                -anchor w \
                -command [list $host ToggleGui $hpanel]]
            pack $btn -side top -fill x
        }
    }
}
em_eventHandler new net_host Ready DisplayHostButtons
em_eventHandler new net_host delete DisplayHostButtons

# Host checkbutton frame
set hframe [frame .omfHosts -bd 4 -relief ridge]
set hpanel [frame .omfPanel -relief groove]

pack $fmb -side left -in $menubar
pack $hmb -side right -in $menubar
pack $menubar -side top -fill x
pack $hframe -side left -fill y
pack $hpanel -side left -fill both -expand 1
update

# Listen for tell messages from all accounts -- they announce new threads
# Each time a new account becomes Ready create a listening Handler for it.
em_eventHandler new net_account Ready StartAccountListener
proc StartAccountListener {} {
    global listeners
    foreach acct [net_account instances] {
        if {![info exists listeners($acct,r)]} {
            set listeners($acct,r) [em_eventHandler new $acct Readable \
                    [list HandleAccountTell $acct]]
            set listeners($acct,d) [em_eventHandler new $acct delete \
                    [list DeleteAccountListener $acct]]
        }
    }
}
proc DeleteAccountListener { acct } {
    global listeners
    if {[info exists listeners($acct,r)]} {
        $listeners($acct,r) delete
        $listeners($acct,d) delete
        unset listeners($acct,r)
        unset listeners($acct,d)
    }
}
proc HandleAccountTell { acct } {
    set message [$acct Get]
    switch -exact -- [lindex $message 0] {
        newthread {
            NewThread $acct
        }
        default {
          [em_extension ApplicationLog] Append "Bad message from account $acct:
	$message"
        }
    }
}
proc NewThread { acct } {
    global qhandler thandler
    if {[info exists qhandler($acct)]} {
        # Only one 'threads' request per account can use qhandler at a time
        after 1000 NewThread $acct
        return
    }
    set qid [$acct Send threads]
    set qhandler($acct) [em_eventHandler new $acct Reply$qid \
            [list NewThreadReply $acct]]
    set thandler($acct) [em_eventHandler new $acct Reply$qid \
            [list NewThreadTimeout $acct]]
}
proc NewThreadReply { acct } {
    global qhandler thandler
    $qhandler($acct) delete
    $thandler($acct) delete
    unset qhandler($acct)
    unset thandler($acct)
    set threads [$acct Get]
    [$acct cget -log] Append "Received thread list: $threads"
    # create a thread for any threads on list not already represented
    # This is a really dumb way to do this!
    if {![lindex $threads 0]} {
        foreach triple [lrange $threads 1 end] {
            net_thread new -hostname [$acct cget -hostname] \
                    -accountname [$acct cget -accountname] \
                    -pid [lindex $triple 1] -alias [lindex $triple 0] \
                    -log [em_extension ApplicationLog]
        }
    }
}
proc NewThreadTimeout { acct } {
    global qhandler thandler
    $qhandler($acct) delete
    $thandler($acct) delete
    unset qhandler($acct)
    unset thandler($acct)
    [$acct cget -log] Append "Timed out waiting for thread list"
}

set a [net_account new -log [em_extension ApplicationLog]]
em_eventHandler new $a delete AccountDied
# This server should answer nothing.  It exists only to register
# the controller
# On second thought -- don't register it
#set s [net_server new -log [em_extension ApplicationLog]]
#$s Start 0

# When any account becomes ready, get its threads
foreach ia [net_account instances] {
    em_eventHandler new $ia Ready [list NewThread $ia]
}

proc bgerror { msg } {
    set log [em_extension ApplicationLog]
    if {[string match em_nop $log]} {
        OMF_NonFatalError $msg
    } else {
        $log Append $msg
    }
}

proc AccountDied {} {
    after 2000 exit
}

proc CleanupAndExit {} {
    global a
    catch {$a delete}
    after 2000 exit
}

OMF_SetIcon .

vwait omfForever

