# FILE: mmpeopendlg.tcl
#
# Open file dialog box for mmProbEd
#

em_class omf_mmpeopendlg {
    const public variable winpath
    const public variable outer_frame_options = "-bd 2 -relief raised"
    const public variable filter = "*.mif"
    const public variable dialog_title = "Load MIF File"
    public variable callback = em_nop
    public variable delete_callback_arg = {}
    private variable filewidget = {}
    private variable selectionwidget = {}
    private variable dirwidget = {}
    constructor {args} {
	eval $this configure -winpath .w$this $args
	# Create toplevel frame to hold dialog
	toplevel $winpath
	wm group $winpath .
	wm title $winpath $dialog_title
	if {![string match {} [info commands OMF_PositionChild]]} {
	    OMF_PositionChild $winpath  ;# Position at +.25+.25 over '.'
	}
	wm protocol $winpath WM_DELETE_WINDOW \
		"$this SwitchBoard $this CANCEL"
	after idle "OMF_SetIcon $winpath"

	# Temporarily disable callbacks until construction is complete
	set callback_keep $callback ;	set callback em_nop

	# Create subwidgets
	set filewidget [omf_fileselect new $winpath \
		-callback "$this SwitchBoard" \
		-filter $filter]
	set ctrlframe [frame $winpath.ctrlframe]
        button $winpath.ctrlframe.rescan -text "Rescan" \
                -command [list $this SwitchBoard $this RESCAN]
        button $winpath.ctrlframe.ok -text "OK" \
                -command [list $this SwitchBoard $this SELECT]
        button $winpath.ctrlframe.cancel -text "Cancel" \
                -command [list $this SwitchBoard $this CANCEL]
	set selectionwidget [omf_selectbox new $winpath \
		-callback "$this SwitchBoard"]
	set dirwidget [omf_dirselect new $winpath \
		-callback "$this SwitchBoard"]

	# Initialize selection path
	$selectionwidget SetPath [$dirwidget cget -value]

	# Pack subwidgets
	pack [$filewidget cget -winpath] -side left \
		-fill both -expand 1 -padx 2m -pady 2m
        pack $winpath.ctrlframe.ok -in $ctrlframe \
                -side right -expand 0 -anchor ne -padx 2m -pady 2m
        pack $winpath.ctrlframe.cancel -in $ctrlframe \
                -side right -expand 0 -anchor ne -padx 2m -pady 2m
        pack $winpath.ctrlframe.rescan -in $ctrlframe \
                -side right -expand 0 -anchor ne -padx 2m -pady 2m
	pack $ctrlframe -side top \
		-fill x -expand 0 -anchor ne -padx 2m -pady 2m
	pack [$selectionwidget cget -winpath] -side top \
		-fill x -expand 0 -anchor n -padx 2m -pady 2m
	pack [$dirwidget cget -winpath] -side right \
		-fill both -expand 1 -padx 2m -pady 2m

	# Widget bindings
	bind $winpath <Destroy> "+$this WinDestroy %W"

	# Re-enable callbacks
	set callback $callback_keep
    }
    method GetFilename {} { selectionwidget } {
	# Returns full pathname of selected value
	return [$selectionwidget ReadFilename]
    }
    method SwitchBoard { subwidget actionid args } \
	    { callback filewidget dirwidget selectionwidget } {
	# General callback routing routine

	switch -exact -- $actionid {
	    UPDATE {
		# Update actions
		if {[string compare $subwidget $filewidget]==0} {
		    $selectionwidget SetFullname [lindex $args 1]
		} elseif {[string compare $subwidget $dirwidget]==0} {
		    $filewidget SetPath [lindex $args 1]
		    $selectionwidget SetPath [lindex $args 1]
		    $selectionwidget SetFilename {}
		} else {
		    error "Unknown UPDATE callback source: $subwidget"
		}
	    }
	    SELECT {
		set errmsg [eval $callback $this OPEN]
		if { [string match {} $errmsg] } {
		    $this delete ; return
		} else {
                    error "ERROR: $errmsg"
                }
	    }
	    CANCEL {
		$this delete ; return
	    }
	    RESCAN {
		$filewidget Rescan
		$dirwidget Rescan
	    }
	    DELETE {}
	    default { error "Unknown actionid: $actionid" }
	}
    }
    method WinDestroy { w } { winpath } {
	if {[info exists winpath] && [string compare $winpath $w]!=0} {
	    return    ;# Child destroy event
	}
	$this delete
    }
    private variable delete_in_progress = 0
    destructor {
	# Protect against re-entrancy
	if { [info exists delete_in_progress] && \
		$delete_in_progress } { return }
	set delete_in_progress 1

	# Notify client of impending destruction
	catch {eval $callback $this DELETE $delete_callback_arg}

	# Delete subwidgets
	catch {$filewidget      delete}
	catch {$selectionwidget delete}
	catch {$dirwidget       delete}

	# Delete window
	if {[info exists winpath] && [winfo exists $winpath]} {
	    # Destroy instance frame, all children, and bindings
	    destroy $winpath
	}
    }
}

